--	Exemplo do uso da interface VGA no kit Mercurio IV

library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.STD_LOGIC_ARITH.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;


entity Labx9 is

   port( 
      CLOCK_50MHz		:	in std_logic;
		KEY				: in std_logic_vector (11 downto 0);
		VGA_R,VGA_G, VGA_B:	out std_logic_vector (3 downto 0);
		VGA_HS, VGA_VS		:	out std_logic

	);
end entity Labx9;

architecture comportamento of Labx9 is
	signal address: std_logic_vector(14 downto 0);
	signal address_sig: std_logic_vector(14 downto 0) := (others => '0');
	signal Pixels,Linha : std_logic_vector(10 downto 0);
	signal RGB, RGB_temp: std_logic_vector(11 downto 0);
	signal video_on: std_logic;
	signal resetn : std_logic;
	

	-- Declaração da Memória
	component memoriavideo
		PORT
		(
			address				: IN STD_LOGIC_VECTOR (14 DOWNTO 0);
			clock					: IN STD_LOGIC;
			q						: OUT STD_LOGIC_VECTOR (11 DOWNTO 0)
		);
	end component memoriavideo;
	
	--	Declaração do controlador da VGA
	component VGA_controlador
		PORT
			( 
					CLOCK_50MHz_i, Reset_n_i	:	in std_logic;
					Video_on_o						:	out std_logic;		
					Pixels_o,Linha_o 				: 	out std_logic_vector(10 downto 0);
					VGA_HS_o,VGA_VS_o				: 	out std_logic       
			);
	end component VGA_controlador;

   -- Declaração sincronizador de reset
   COMPONENT reset_sync
      PORT
      (
         i_clk	               :	 IN STD_LOGIC;
         i_external_reset_n	:	 IN STD_LOGIC;
         o_reset_n      		:	 OUT STD_LOGIC;
         o_reset		         :	 OUT STD_LOGIC
      );
   END COMPONENT;   
   
   
	begin
		-- O sistema é resetado quando resetn está em nivel lógico baixo
		-- por isso o valor da chave é negado (chave pressionada = '1')
		--resetn <= not KEY(11);
		reset_synch_50mhz_inst : reset_sync
      PORT MAP (
         i_clk                => CLOCK_50MHz,
         i_external_reset_n   => KEY(11),
         o_reset		         => resetn,
         o_reset_n            => open
      );

      
      
		-- Instanciação da Memória
		memoriavideo_inst : memoriavideo PORT MAP (
				address	 => address,
				clock	 => CLOCK_50MHz,
				q	 => RGB_temp
			);
		
		-- Instanciação do conrolador da VGA
		VGA_controlador_inst : VGA_controlador port map (
				CLOCK_50MHz_i => CLOCK_50MHz,
				Reset_n_i => resetn,
				Video_on_o => Video_on,
				Pixels_o => Pixels,
				Linha_o => Linha,
				VGA_HS_o => VGA_HS,
				VGA_VS_o => VGA_VS
		);
		
		-- Processo que controla as saídas RGB
		VGA_RGB:
		process (CLOCK_50MHz,resetn)
		begin
			if resetn = '0' then
				VGA_R	<=	"0000";
				VGA_G	<=	"0000";
				VGA_B <= "0000";
			elsif rising_edge (CLOCK_50MHz) then
				if (Video_on = '0') Then
					VGA_R	<=	"0000";
					VGA_G	<=	"0000";
					VGA_B <= "0000";				
				else
					VGA_R 	<=	RGB(11 downto 8);
					VGA_G		<=	RGB( 7 downto 4);
					VGA_B		<=	RGB( 3 downto 0);
				end if;
			end if;
		end Process VGA_RGB;
		
		-- Endereçamento da memória
		Enderecamento: process(Linha, Pixels)
			begin
				-- Enviando endereços para a memória. O decremento na linha e na coluna
				-- serve para indicar o endereço correto, pois a imagem está deslocada
				Address <= (Linha(6 downto 0)- "0101111") & (Pixels(7 downto 0) - "10111111");
				-- Nos pixels onde não tem imagem serão cpreenchido com a cor azul
				RGB <= "000001010110";
				if (Pixels > 191 and Pixels < 446) and (Linha >175 and linha < 302) then
				-- O numeros definem a região onde a imagem será mostrada (a imagem armazenada 
				--	na memória é de 256 x 128)
					RGB <= RGB_temp;
				end if;
			end Process Enderecamento;

end comportamento;